@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2002, 2003, 2004 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -8,28 +8,33 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 #include <gtk/gtk.h>
 
 #include <gpe/errorbox.h>
 #include <gpe/pixmaps.h>
 #include <gpe/gtkdatecombo.h>
 #include <gpe/picturebutton.h>
+#include <gpe/pim-categories-ui.h>
 
-#include "interface.h"
 #include "support.h"
 #include "structure.h"
-#include "callbacks.h"
 #include "db.h"
 
+void on_edit_cancel_clicked (GtkButton * button, gpointer user_data);
+void on_edit_save_clicked (GtkButton * button, gpointer user_data);
+void on_edit_bt_image_clicked (GtkButton * button, gpointer user_data);
+void on_categories_clicked (GtkButton *button, gpointer user_data);
+
 static void
 add_tag (gchar *tag, GtkWidget *w, GtkWidget *pw)
 {
   GSList *tags;
-  gtk_object_set_data (GTK_OBJECT (w), "db-tag", tag);
+  g_object_set_data (G_OBJECT (w), "db-tag", tag);
 
   tags = gtk_object_get_data (GTK_OBJECT (pw), "tag-widgets");
   tags = g_slist_append (tags, w);
-  gtk_object_set_data (GTK_OBJECT (pw), "tag-widgets", tags);
+  g_object_set_data (G_OBJECT (pw), "tag-widgets", tags);
 }
 
 static void
@@ -130,12 +135,6 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
   singles = NULL;
 }
 
-static void
-destroy_category_widgets (gpointer p)
-{
-  g_slist_free ((GSList *)p);
-}
-
 static GtkWidget*
 create_edit (void)
 {
@@ -151,17 +150,17 @@ create_edit (void)
   GtkWidget *edit_bt_bdate;
   GtkWidget *hbox4;
   GtkWidget *edit_bt_image;
-  GtkWidget *scrolledwindow3;
-  GtkWidget *cbox;
+  GtkWidget *cathbox;
   GtkWidget *name_entry;
   GtkWidget *summary_entry;
   GtkWidget *label16;
   GtkWidget *edit_cancel;
   GtkWidget *edit_save;
-  GtkWidget *catframe;
+  GtkWidget *edit_delete;
+  GtkWidget *catlabel, *catbutton;
   GtkTooltips *tooltips;
   GtkWidget *topvbox;
-  GSList *categories = db_get_categories ();
+  GtkWidget *topscrolledwindow;
   GtkWidget *vbox, *action_area;
 
   tooltips = gtk_tooltips_new ();
@@ -185,8 +184,15 @@ create_edit (void)
   gtk_box_pack_start (GTK_BOX (vbox), notebook2, 
 		      TRUE, TRUE, 0);
 
+  topscrolledwindow = gtk_scrolled_window_new (NULL, NULL);
+
   topvbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (notebook2), topvbox);
+
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (topscrolledwindow),
+				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (topscrolledwindow), topvbox);
+
+  gtk_container_add (GTK_CONTAINER (notebook2), topscrolledwindow/*topvbox*/);
 
   table1 = gtk_table_new (4, 2, FALSE);
   gtk_box_pack_start (GTK_BOX (topvbox), table1, FALSE, FALSE, 2);
@@ -228,7 +234,6 @@ create_edit (void)
   GTK_WIDGET_UNSET_FLAGS (datecombo, GTK_CAN_DEFAULT);
 
   edit_bt_bdate = gtk_check_button_new_with_label (_("Schedule"));
-  gtk_widget_set_name (edit_bt_bdate, "edit_bt_bdate");
   gtk_widget_set_sensitive (edit_bt_bdate, FALSE);		/* XXX */
   gtk_box_pack_start (GTK_BOX (hbox3), edit_bt_bdate, TRUE, TRUE, 0);
   gtk_tooltips_set_tip (tooltips, edit_bt_bdate, _("automatic appointment"), NULL);
@@ -242,36 +247,15 @@ create_edit (void)
   gtk_box_pack_start (GTK_BOX (hbox4), edit_bt_image, TRUE, TRUE, 0);
   gtk_tooltips_set_tip (tooltips, edit_bt_image, _("click to choose file"), NULL);
 
-  catframe = gtk_frame_new (_("Categories"));
-  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (catframe), scrolledwindow3);
-  gtk_box_pack_start (GTK_BOX (topvbox), catframe, TRUE, TRUE, 2);
+  cathbox = gtk_hbox_new (FALSE, 0);
+  catbutton = gtk_button_new_with_label (_("Categories:"));
+  catlabel = gtk_label_new (NULL);
 
-  cbox = gtk_vbox_new (FALSE, 0);
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindow3), 
-					 cbox);
-
-  if (categories)
-    {
-      GSList *iter;
-      GSList *category_widgets = NULL;
-      for (iter = categories; iter; iter = iter->next)
-	{
-	  struct category *c = iter->data;
-	  GtkWidget *w = gtk_check_button_new_with_label (c->name);
-	  gtk_object_set_data (GTK_OBJECT (w), "category", (gpointer)c->id);
-	  gtk_box_pack_start (GTK_BOX (cbox), w, FALSE, FALSE, 0);
-	  g_free (c->name);
-	  g_free (c);
-	  category_widgets = g_slist_append (category_widgets, w);
-	}
-      
-      gtk_object_set_data_full (GTK_OBJECT (edit), "category-widgets", category_widgets,
-				(GtkDestroyNotify) destroy_category_widgets);
-
-      g_slist_free (categories);
-    }
+  gtk_box_pack_start (GTK_BOX (cathbox), catbutton, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (cathbox), catlabel, TRUE, TRUE, 4);
+  gtk_misc_set_alignment (GTK_MISC (catlabel), 0.0, 0.5);  
+  
+  gtk_box_pack_start (GTK_BOX (topvbox), cathbox, FALSE, FALSE, 2);
 
   name_entry = gtk_entry_new ();
   add_tag ("NAME", name_entry, edit);
@@ -287,28 +271,33 @@ create_edit (void)
   label16 = gtk_label_new (_("Personal"));
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 0), label16);
 
-  edit_cancel = gpe_button_new_from_stock (GTK_STOCK_CANCEL, GPE_BUTTON_TYPE_BOTH);
-  gtk_container_add (GTK_CONTAINER (action_area), edit_cancel);
-  GTK_WIDGET_SET_FLAGS (edit_cancel, GTK_CAN_DEFAULT);
-
-  edit_save = gpe_button_new_from_stock (GTK_STOCK_SAVE, GPE_BUTTON_TYPE_BOTH);
-  gtk_container_add (GTK_CONTAINER (action_area), edit_save);
+  edit_delete = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+  edit_cancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+  edit_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
   GTK_WIDGET_SET_FLAGS (edit_save, GTK_CAN_DEFAULT);
 
-  gtk_signal_connect (GTK_OBJECT (edit_bt_image), "clicked",
-                      GTK_SIGNAL_FUNC (on_edit_bt_image_clicked),
-                      NULL);
-  gtk_signal_connect (GTK_OBJECT (edit_cancel), "clicked",
-                      GTK_SIGNAL_FUNC (on_edit_cancel_clicked),
-                      edit);
-  gtk_signal_connect (GTK_OBJECT (edit_save), "clicked",
-                      GTK_SIGNAL_FUNC (on_edit_save_clicked),
-                      edit);
+  gtk_widget_set_sensitive (GTK_WIDGET (edit_delete), FALSE);
+
+  gtk_box_pack_start (GTK_BOX (action_area), edit_delete, TRUE, FALSE, 4);
+  gtk_box_pack_start (GTK_BOX (action_area), edit_cancel, TRUE, FALSE, 4);
+  gtk_box_pack_start (GTK_BOX (action_area), edit_save, TRUE, FALSE, 4);
+
+  g_signal_connect (G_OBJECT (edit_bt_image), "clicked",
+		    G_CALLBACK (on_edit_bt_image_clicked), NULL);
+  g_signal_connect (G_OBJECT (edit_cancel), "clicked",
+		    G_CALLBACK (on_edit_cancel_clicked), edit);
+  g_signal_connect (G_OBJECT (edit_save), "clicked",
+		    G_CALLBACK (on_edit_save_clicked), edit);
+  g_signal_connect (G_OBJECT (catbutton), "clicked",
+		    G_CALLBACK (on_categories_clicked), edit);
 
   g_object_set_data (G_OBJECT (edit), "tooltips", tooltips);
   g_object_set_data (G_OBJECT (edit), "notebook2", notebook2);
   g_object_set_data (G_OBJECT (edit), "name_entry", name_entry);
   g_object_set_data (G_OBJECT (edit), "datecombo", datecombo);
+  g_object_set_data (G_OBJECT (edit), "categories-label", catlabel);
+
+  gtk_window_set_default_size (GTK_WINDOW (edit), 240, 320);
 
   return edit;
 }
@@ -348,3 +337,253 @@ edit_window (void)
   return w;
 }
 
+void
+retrieve_special_fields (GtkWidget * edit, struct person *p)
+{
+  GSList *cl = g_object_get_data (G_OBJECT (edit), "category-widgets");
+  db_delete_tag (p, "CATEGORY");
+  while (cl)
+    {
+      GtkWidget *w = cl->data;
+      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
+	{
+	  guint c = (guint) g_object_get_data (G_OBJECT (w), "category");
+	  char buf[32];
+	  snprintf (buf, sizeof (buf) - 1, "%d", c);
+	  buf[sizeof (buf) - 1] = 0;
+	  db_set_multi_data (p, "CATEGORY", g_strdup (buf));
+	}
+      cl = cl->next;
+    }
+
+  db_delete_tag (p, "BIRTHDAY");
+  {
+    GtkDateCombo *c = GTK_DATE_COMBO (lookup_widget (edit, "datecombo"));
+    if (c->set)
+      {
+	char buf[32];
+	snprintf (buf, sizeof (buf) - 1, "%04d%02d%02d", c->year, c->month,
+		  c->day);
+	buf[sizeof (buf) - 1] = 0;
+	db_set_data (p, "BIRTHDAY", g_strdup (buf));
+      }
+  }
+}
+
+static GSList *
+get_categories_list (struct person *p)
+{
+  GSList *iter;
+  GSList *list = NULL;
+
+  iter = p->data;
+
+  for (; iter; iter = iter->next)
+    {
+      struct tag_value *t = iter->data;
+      if (!strcasecmp (t->tag, "category"))
+	{
+	  int id = atoi (t->value);
+
+	  list = g_slist_prepend (list, (gpointer)id);
+	}
+    }
+
+  return list;
+}
+
+static gchar *
+build_categories_string (struct person *p)
+{
+  gchar *s = NULL;
+  GSList *iter, *list;
+
+  list = get_categories_list (p);
+
+  for (iter = list; iter; iter = iter->next)
+    {
+      const gchar *cat;
+      cat = gpe_pim_category_name ((int)iter->data);
+
+      if (cat)
+	{
+	  if (s)
+	    {
+	      char *ns = g_strdup_printf ("%s, %s", s, cat);
+	      g_free (s);
+	      s = ns;
+	    }
+	  else
+	    s = g_strdup (cat);
+	}
+    }
+
+  g_slist_free (list);
+
+  return s;
+}
+
+static void
+update_categories_list (GtkWidget *ui, GSList *selected, GtkWidget *edit)
+{
+  struct person *p;
+  gchar *str;
+  GtkWidget *w;
+  GSList *iter;
+
+  p = g_object_get_data (G_OBJECT (edit), "person");
+
+  db_delete_tag (p, "category");
+
+  for (iter = selected; iter; iter = iter->next)
+    {
+      db_set_multi_data (p, "category", g_strdup_printf ("%d", (int)iter->data));
+    }
+
+  str = build_categories_string (p);
+  
+  w = lookup_widget (edit, "categories-label");
+  gtk_label_set_text (GTK_LABEL (w), str);
+  g_free (str);
+}
+
+static void
+store_special_fields (GtkWidget *edit, struct person *p)
+{
+  GtkWidget *w;
+  struct tag_value *v;
+  gchar *str;
+
+  w = lookup_widget (edit, "datecombo");
+  v = p ? db_find_tag (p, "BIRTHDAY") : NULL;
+  if (v && v->value)
+    {
+      guint year, month, day;
+      sscanf (v->value, "%04d%02d%02d", &year, &month, &day);
+      gtk_date_combo_set_date (GTK_DATE_COMBO (w), year, month, day);
+    }
+  else
+    gtk_date_combo_clear (GTK_DATE_COMBO (w));
+
+  if (p)
+    {
+      str = build_categories_string (p);
+      if (str)
+	{
+	  w = lookup_widget (edit, "categories-label");
+	  gtk_label_set_text (GTK_LABEL (w), str);
+	  g_free (str);
+	}
+    }
+}
+
+void
+edit_person (struct person *p)
+{
+  GtkWidget *w = edit_window ();
+  GtkWidget *entry = lookup_widget (w, "name_entry");
+  if (p)
+    {
+      GSList *tags = gtk_object_get_data (GTK_OBJECT (w), "tag-widgets");
+      GSList *iter;
+      for (iter = tags; iter; iter = iter->next)
+	{
+	  GtkWidget *w = iter->data;
+	  gchar *tag = gtk_object_get_data (GTK_OBJECT (w), "db-tag");
+	  struct tag_value *v = db_find_tag (p, tag);
+	  guint pos = 0;
+	  if (v && v->value)
+	    {
+	      if (GTK_IS_EDITABLE (w))
+		gtk_editable_insert_text (GTK_EDITABLE (w), v->value,
+					  strlen (v->value), &pos);
+	      else
+		gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (w)), 
+					  v->value, -1);
+	    }
+	}
+      gtk_object_set_data (GTK_OBJECT (w), "person", p);
+    }
+  store_special_fields (w, p);
+  gtk_widget_show_all (w);
+  gtk_widget_grab_focus (entry);
+}
+
+void
+on_edit_save_clicked (GtkButton * button, gpointer user_data)
+{
+  GtkWidget *edit = (GtkWidget *) user_data;
+  GSList *tags;
+  struct person *p = g_object_get_data (G_OBJECT (edit), "person");
+  
+  for (tags = g_object_get_data (G_OBJECT (edit), "tag-widgets");
+       tags; tags = tags->next)
+    {
+      GtkWidget *w = tags->data;
+      gchar *text, *tag;
+      if (GTK_IS_EDITABLE (w))
+	text = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+      else
+	{
+	  GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (w));
+	  GtkTextIter start, end;
+	  gtk_text_buffer_get_bounds (buf, &start, &end);
+	  text = gtk_text_buffer_get_text (buf, &start, &end, FALSE);
+	}
+      tag = g_object_get_data (G_OBJECT (w), "db-tag");
+      db_set_data (p, tag, text);
+    }
+
+  retrieve_special_fields (edit, p);
+
+  if (commit_person (p))
+    {
+      gtk_widget_destroy (edit);
+      discard_person (p);
+      update_display ();
+    }
+}
+
+void
+on_categories_clicked (GtkButton *button, gpointer user_data)
+{
+  struct person *p;
+  GtkWidget *w;
+
+  p = g_object_get_data (G_OBJECT (user_data), "person");
+
+  w = gpe_pim_categories_dialog (get_categories_list (p), G_CALLBACK (update_categories_list), user_data);
+}
+
+void 
+store_filename (GtkWidget * w, GtkFileSelection * selector)
+{
+  const gchar *selected_filename =
+    gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector));
+}
+
+void
+on_edit_bt_image_clicked (GtkButton * button, gpointer user_data)
+{
+  GtkWidget *filesel = gtk_file_selection_new ("Select image");
+
+  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
+		      "clicked", G_CALLBACK (store_filename), filesel);
+
+  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
+		    "clicked", G_CALLBACK (gtk_widget_destroy),
+		    (gpointer) filesel);
+
+  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
+		    "clicked", G_CALLBACK (gtk_widget_destroy),
+		    (gpointer) filesel);
+
+  gtk_widget_show_all (filesel);
+}
+
+void
+on_edit_cancel_clicked (GtkButton * button, gpointer user_data)
+{
+  gtk_widget_destroy (GTK_WIDGET (user_data));
+}
+
